@@ -15,8 +15,8 @@ from .solvers.fftsolvers import FFTSolver3D
 from .beam_elements.spacecharge import SpaceCharge3D, SpaceChargeBiGaussian
 from .beam_elements.beambeam2d import BeamBeamBiGaussian2D
 from .beam_elements.beambeam3d import BeamBeamBiGaussian3D
-from .beam_elements.beambeam3d import TempSlicer
 from .beam_elements.beambeam3d import ConfigForUpdateBeamBeamBiGaussian3D
+from .beam_elements.temp_slicer import TempSlicer
 from .beam_elements.electroncloud import ElectronCloud
 from .beam_elements.electronlens_interpolated import ElectronLensInterpolated
 
@@ -27,6 +27,8 @@ from .config_tools import configure_orbit_dependent_parameters_for_bb
 from .config_tools import install_spacecharge_frozen
 from .config_tools import full_electroncloud_setup
 
+from ._version import __version__
+
 
 import xtrack as _xt
 
