@@ -7,9 +7,28 @@ import numpy as np
 
 import xobjects as xo
 import xtrack as xt
+import xpart as xp
 
 from ..general import _pkg_root
 
+class BeamstrahlungTable(xo.HybridClass):
+    _xofields = {
+      '_index': xt.RecordIndex,
+      'at_element': xo.Int64[:],
+      'at_turn': xo.Int64[:],
+      'particle_id': xo.Int64[:],
+      'photon_id': xo.Float64[:],
+      'photon_energy': xo.Float64[:],
+      'photon_critical_energy': xo.Float64[:],
+      'primary_energy': xo.Float64[:],
+      'rho_inv': xo.Float64[:],
+        }
+
+class BeamBeamBiGaussian3DRecord(xo.HybridClass):
+    _xofields = {
+        'beamstrahlungtable': BeamstrahlungTable,
+       }
+
 class BeamBeamBiGaussian3D(xt.BeamElement):
 
     _xofields = {
@@ -70,21 +89,37 @@ class BeamBeamBiGaussian3D(xt.BeamElement):
         'min_sigma_diff': xo.Float64,
         'threshold_singular': xo.Float64,
 
+        # beamstrahlung
+        'flag_beamstrahlung': xo.Int64,
+        'slices_other_beam_zeta_bin_width_star_beamstrahlung': xo.Float64[:],
+        'other_beam_sigma_55_star_beamstrahlung': xo.Float64,
+
     }
 
+    _internal_record_class = BeamBeamBiGaussian3DRecord
+
+    _rename = {'flag_beamstrahlung': '_flag_beamstrahlung'}
+
     _extra_c_sources= [
         _pkg_root.joinpath('headers/constants.h'),
         _pkg_root.joinpath('headers/sincos.h'),
         _pkg_root.joinpath('headers/power_n.h'),
-        _pkg_root.joinpath('fieldmaps/bigaussian_src/complex_error_function.h'),
+        _pkg_root.joinpath('fieldmaps/bigaussian_src/faddeeva.h'),
         '#define NOFIELDMAP', #TODO Remove this workaround
         _pkg_root.joinpath('fieldmaps/bigaussian_src/bigaussian.h'),
         '#undef NOFIELDMAP', #TODO Remove this workaround
         _pkg_root.joinpath('beam_elements/beambeam_src/beambeam3d_transport_sigmas.h'),
         _pkg_root.joinpath('beam_elements/beambeam_src/beambeam3d_ref_frame_changes.h'),
+
+        # beamstrahlung
+        xp.general._pkg_root.joinpath('random_number_generator/rng_src/base_rng.h'),
+        xp.general._pkg_root.joinpath('random_number_generator/rng_src/local_particle_rng.h'),
+        _pkg_root.joinpath('headers/beamstrahlung_spectrum.h'),
+
         _pkg_root.joinpath('beam_elements/beambeam_src/beambeam3d.h'),
         _pkg_root.joinpath('beam_elements/beambeam_src/beambeam3d_methods_for_strongstrong.h'),
-    ]
+
+   ]
 
     _per_particle_kernels={
         'synchro_beam_kick': xo.Kernel(
@@ -101,7 +136,7 @@ class BeamBeamBiGaussian3D(xt.BeamElement):
     }
 
     def __init__(self,
-                    phi=None, alpha=None, other_beam_q0=None, particles_per_macroparticle = None,
+                    phi=None, alpha=None, other_beam_q0=None,
                     scale_strength = 1.,
 
                     slices_other_beam_num_particles=None,
@@ -113,6 +148,11 @@ class BeamBeamBiGaussian3D(xt.BeamElement):
                     slices_other_beam_zeta_center=None,
                     slices_other_beam_pzeta_center=0.,
 
+                    flag_beamstrahlung=0,
+
+                    slices_other_beam_zeta_bin_width_star_beamstrahlung=None,
+                    other_beam_sigma_55_star_beamstrahlung=None,
+
                     slices_other_beam_x_center_star=None,
                     slices_other_beam_px_center_star=None,
                     slices_other_beam_y_center_star=None,
@@ -183,27 +223,36 @@ class BeamBeamBiGaussian3D(xt.BeamElement):
             self.iscollective = True
             self.track = self._track_collective # switch to specific track method
 
-            #assert slices_other_beam_zeta_center is not None
-            #assert not np.isscalar(slices_other_beam_num_particles)
-
-            if slices_other_beam_zeta_center is None: 
-                slices_other_beam_zeta_center = self.config_for_update.slicer.bin_centers
+            if slices_other_beam_zeta_center is None:
+                    slices_other_beam_zeta_center = self.config_for_update.slicer.bin_centers
             # Some dummy values just to initialize the object
-            if slices_other_beam_Sigma_11 is None: slices_other_beam_Sigma_11 = 1.
-            if slices_other_beam_Sigma_12 is None: slices_other_beam_Sigma_12 = 1.
-            if slices_other_beam_Sigma_22 is None: slices_other_beam_Sigma_22 = 1.
-            if slices_other_beam_Sigma_33 is None: slices_other_beam_Sigma_33 = 1.
-            if slices_other_beam_Sigma_34 is None: slices_other_beam_Sigma_34 = 1.
-            if slices_other_beam_Sigma_44 is None: slices_other_beam_Sigma_44 = 1.
+            if (slices_other_beam_Sigma_11 is None
+                    and slices_other_beam_Sigma_11_star is None):
+                slices_other_beam_Sigma_11 = 1.
+            if (slices_other_beam_Sigma_12 is None
+                    and slices_other_beam_Sigma_12_star is None):
+                slices_other_beam_Sigma_12 = 0.
+            if (slices_other_beam_Sigma_22 is None
+                    and slices_other_beam_Sigma_22_star is None):
+                slices_other_beam_Sigma_22 = 1.
+            if (slices_other_beam_Sigma_33 is None
+                    and slices_other_beam_Sigma_33_star is None):
+                slices_other_beam_Sigma_33 = 1.
+            if (slices_other_beam_Sigma_34 is None
+                    and slices_other_beam_Sigma_34_star is None):
+                slices_other_beam_Sigma_34 = 0.
+            if (slices_other_beam_Sigma_44 is None
+                    and slices_other_beam_Sigma_44_star is None):
+                slices_other_beam_Sigma_44 = 1.
+
             if slices_other_beam_num_particles is None:
                 slices_other_beam_num_particles = np.zeros_like(
                                             slices_other_beam_zeta_center)
-
             self.moments = None
-            self.partner_moments = np.zeros(self.config_for_update.slicer.num_slices*(1+6+10),dtype=float)
-
-            self.particles_per_macroparticle = particles_per_macroparticle
-
+            self.partner_moments = np.zeros(
+                self.config_for_update.slicer.num_slices*(1+6+10), dtype=float)
+        else:
+            self.config_for_update = None
 
         if old_interface is not None:
             self._init_from_old_interface(old_interface=old_interface, **kwargs)
@@ -232,11 +281,6 @@ class BeamBeamBiGaussian3D(xt.BeamElement):
 
         self._allocate_xobject(n_slices, **kwargs)
 
-        if self.iscollective:
-            if not isinstance(self._buffer.context, xo.ContextCpu):
-                raise NotImplementedError(
-                    'BeamBeamBiGaussian3D only works with CPU context for now')
-
         if phi is None:
             assert _sin_phi is not None and _cos_phi is not None and _tan_phi is not None, (
                 'phi must be specified if _sin_phi, _cos_phi, _tan_phi are not')
@@ -277,6 +321,7 @@ class BeamBeamBiGaussian3D(xt.BeamElement):
 
         # Initialize slice positions in the boosted frame
         self._init_starred_positions(
+            slices_other_beam_num_particles,
             slices_other_beam_x_center, slices_other_beam_px_center,
             slices_other_beam_y_center, slices_other_beam_py_center,
             slices_other_beam_zeta_center, slices_other_beam_pzeta_center,
@@ -312,6 +357,32 @@ class BeamBeamBiGaussian3D(xt.BeamElement):
         self.min_sigma_diff = min_sigma_diff
         self.threshold_singular = threshold_singular
 
+        # beamstrahlung
+        if self.config_for_update is not None:
+            if (slices_other_beam_zeta_bin_width_star_beamstrahlung is None):
+                    slices_other_beam_zeta_bin_width_star_beamstrahlung = (
+                        self.config_for_update.slicer.bin_widths_beamstrahlung
+                            / np.cos(self.phi))
+        if slices_other_beam_zeta_bin_width_star_beamstrahlung is not None:
+            assert not np.isscalar(slices_other_beam_zeta_bin_width_star_beamstrahlung), (
+                'slices_other_beam_zeta_bin_width_star_beamstrahlung must be an array')
+            assert (len(self.slices_other_beam_zeta_bin_width_star_beamstrahlung)
+                == len(self.slices_other_beam_num_particles))
+            self.slices_other_beam_zeta_bin_width_star_beamstrahlung = self._arr2ctx(
+                np.array(slices_other_beam_zeta_bin_width_star_beamstrahlung))
+        else:
+            self.slices_other_beam_zeta_bin_width_star_beamstrahlung[:] = 0
+
+        if other_beam_sigma_55_star_beamstrahlung:
+            self.other_beam_sigma_55_star_beamstrahlung = (
+                                        other_beam_sigma_55_star_beamstrahlung)
+        else:
+            self.other_beam_sigma_55_star_beamstrahlung = 0
+
+        # beamstrahlung
+
+        self.flag_beamstrahlung = flag_beamstrahlung # Trigger property setter
+
     def _allocate_xobject(self, n_slices, **kwargs):
         self.xoinitialize(
             slices_other_beam_Sigma_11_star=n_slices,
@@ -331,9 +402,32 @@ class BeamBeamBiGaussian3D(xt.BeamElement):
             slices_other_beam_py_center_star=n_slices,
             slices_other_beam_zeta_center_star=n_slices,
             slices_other_beam_pzeta_center_star=n_slices,
+            slices_other_beam_zeta_bin_width_star_beamstrahlung=n_slices,  # beamstrahlung
             **kwargs
             )
 
+    @property
+    def flag_beamstrahlung(self):
+        return self._flag_beamstrahlung
+
+    @flag_beamstrahlung.setter
+    def flag_beamstrahlung(self, flag_beamstrahlung):
+        # beamstrahlung
+        if flag_beamstrahlung == 1:
+            raise NotImplementedError('"mean" model not yet finished')
+            if self.other_beam_sigma_55_star_beamstrahlung == 0 :
+                raise ValueError(
+                    'other_beam_sigma_55_star_beamstrahlung '
+                    'needs to be correctly set')
+        elif flag_beamstrahlung == 2:
+            ctx2np  = self._context.nparray_from_context_array
+            if np.any(ctx2np(
+                self.slices_other_beam_zeta_bin_width_star_beamstrahlung == 0)):
+                raise ValueError(
+                    'slices_other_beam_zeta_bin_width_star_beamstrahlung '
+                    'needs to be correctly set')
+        self._flag_beamstrahlung = flag_beamstrahlung
+
     def _init_from_old_interface(self, old_interface, **kwargs):
 
         params=old_interface
@@ -522,9 +616,8 @@ class BeamBeamBiGaussian3D(xt.BeamElement):
                                                      particles.at_turn[0],
                                                      internal_tag=self.config_for_update._i_step):
                     # Compute moments
-                    self.config_for_update.slicer.assign_slices(particles)
+                    self.config_for_update.slicer.assign_slices(particles)  # in this the bin edges are fixed with TempSlicer
                     self.moments = self.config_for_update.slicer.compute_moments(particles,update_assigned_slices=False)
-                    self.moments[:self.config_for_update.slicer.num_slices] *= self.particles_per_macroparticle
                     self.config_for_update.pipeline_manager.send_message(self.moments,
                                                      self.config_for_update.element_name,
                                                      particles.name,
@@ -545,15 +638,16 @@ class BeamBeamBiGaussian3D(xt.BeamElement):
                 else:
                     return xt.PipelineStatus(on_hold=True)
 
+            # compute interacting other beam slice ID
             self.config_for_update._other_beam_slice_index_for_particles[:] =(
                  self.config_for_update._i_step - self.config_for_update._particles_slice_index)
-            self.config_for_update._other_beam_slice_index_for_particles[
-                             self.config_for_update._particles_slice_index < 0] = -1
+
             self.synchro_beam_kick(particles=particles,
                         i_slice_for_particles=self.config_for_update._other_beam_slice_index_for_particles)
 
             self.config_for_update._i_step += 1
-            if self.config_for_update._i_step == (n_slices_self_beam + self.num_slices_other_beam):
+            if self.config_for_update._i_step == (
+                            n_slices_self_beam + self.num_slices_other_beam - 1):
                 self.config_for_update._i_step = 0
                 self.config_for_update._working_on_bunch = None
                 break
@@ -653,18 +747,24 @@ class BeamBeamBiGaussian3D(xt.BeamElement):
             ):
 
         # Mandatory sigmas
-        assert (slices_other_beam_Sigma_11 is not None or slices_other_beam_Sigma_11_star is not None), (
-            "`slices_other_beam_Sigma_11` must be provided")
-        assert (slices_other_beam_Sigma_12 is not None or slices_other_beam_Sigma_12_star is not None), (
-            "`slices_other_beam_Sigma_12` must be provided")
-        assert (slices_other_beam_Sigma_22 is not None or slices_other_beam_Sigma_22_star is not None), (
-            "`slices_other_beam_Sigma_22` must be provided")
-        assert (slices_other_beam_Sigma_33 is not None or slices_other_beam_Sigma_33_star is not None), (
-            "`slices_other_beam_Sigma_33` must be provided")
-        assert (slices_other_beam_Sigma_34 is not None or slices_other_beam_Sigma_34_star is not None), (
-            "`slices_other_beam_Sigma_34` must be provided")
-        assert (slices_other_beam_Sigma_44 is not None or slices_other_beam_Sigma_44_star is not None), (
-            "`slices_other_beam_Sigma_44` must be provided")
+        assert ((slices_other_beam_Sigma_11 is None or slices_other_beam_Sigma_11_star is not None)
+                or (slices_other_beam_Sigma_11 is not None or slices_other_beam_Sigma_11_star is None)), (
+            "Either `slices_other_beam_Sigma_11` or `slices_other_beam_Sigma_11_star` must be provided")
+        assert ((slices_other_beam_Sigma_12 is None or slices_other_beam_Sigma_12_star is not None)
+                or (slices_other_beam_Sigma_12 is not None or slices_other_beam_Sigma_12_star is None)), (
+            "Either `slices_other_beam_Sigma_12` or `slices_other_beam_Sigma_12_star` must be provided")
+        assert ((slices_other_beam_Sigma_22 is None or slices_other_beam_Sigma_22_star is not None)
+                or (slices_other_beam_Sigma_22 is not None or slices_other_beam_Sigma_22_star is None)), (
+            "Either `slices_other_beam_Sigma_22` or `slices_other_beam_Sigma_22_star` must be provided")
+        assert ((slices_other_beam_Sigma_33 is None or slices_other_beam_Sigma_33_star is not None)
+                or (slices_other_beam_Sigma_33 is not None or slices_other_beam_Sigma_33_star is None)), (
+            "Either `slices_other_beam_Sigma_33` or `slices_other_beam_Sigma_33_star` must be provided")
+        assert ((slices_other_beam_Sigma_34 is None or slices_other_beam_Sigma_34_star is not None)
+                or (slices_other_beam_Sigma_34 is not None or slices_other_beam_Sigma_34_star is None)), (
+            "Either `slices_other_beam_Sigma_34` or `slices_other_beam_Sigma_34_star` must be provided")
+        assert ((slices_other_beam_Sigma_44 is None or slices_other_beam_Sigma_44_star is not None)
+                or (slices_other_beam_Sigma_44 is not None or slices_other_beam_Sigma_44_star is None)), (
+            "Either `slices_other_beam_Sigma_44` or `slices_other_beam_Sigma_44_star` must be provided")
 
         # Coupling between transverse planes
         if slices_other_beam_Sigma_13 is None and slices_other_beam_Sigma_13_star is None:
@@ -727,6 +827,7 @@ class BeamBeamBiGaussian3D(xt.BeamElement):
             self.slices_other_beam_Sigma_44_star = self._arr2ctx(slices_other_beam_Sigma_44_star)
 
     def _init_starred_positions(self,
+            slices_other_beam_num_particles,
             slices_other_beam_x_center,
             slices_other_beam_px_center,
             slices_other_beam_y_center,
@@ -740,11 +841,38 @@ class BeamBeamBiGaussian3D(xt.BeamElement):
             slices_other_beam_zeta_center_star,
             slices_other_beam_pzeta_center_star):
 
+        if isinstance(slices_other_beam_num_particles, (list, tuple)):
+            slices_other_beam_num_particles = np.array(slices_other_beam_num_particles)
+        if isinstance(slices_other_beam_x_center, (list, tuple)):
+            slices_other_beam_x_center = np.array(slices_other_beam_x_center)
+        if isinstance(slices_other_beam_px_center, (list, tuple)):
+            slices_other_beam_px_center = np.array(slices_other_beam_px_center)
+        if isinstance(slices_other_beam_y_center, (list, tuple)):
+            slices_other_beam_y_center = np.array(slices_other_beam_y_center)
+        if isinstance(slices_other_beam_py_center, (list, tuple)):
+            slices_other_beam_py_center = np.array(slices_other_beam_py_center)
+        if isinstance(slices_other_beam_zeta_center, (list, tuple)):
+            slices_other_beam_zeta_center = np.array(slices_other_beam_zeta_center)
+        if isinstance(slices_other_beam_pzeta_center, (list, tuple)):
+            slices_other_beam_pzeta_center = np.array(slices_other_beam_pzeta_center)
+        if isinstance(slices_other_beam_x_center_star, (list, tuple)):
+            slices_other_beam_x_center_star = np.array(slices_other_beam_x_center_star)
+        if isinstance(slices_other_beam_px_center_star, (list, tuple)):
+            slices_other_beam_px_center_star = np.array(slices_other_beam_px_center_star)
+        if isinstance(slices_other_beam_y_center_star, (list, tuple)):
+            slices_other_beam_y_center_star = np.array(slices_other_beam_y_center_star)
+        if isinstance(slices_other_beam_py_center_star, (list, tuple)):
+            slices_other_beam_py_center_star = np.array(slices_other_beam_py_center_star)
+        if isinstance(slices_other_beam_zeta_center_star, (list, tuple)):
+            slices_other_beam_zeta_center_star = np.array(slices_other_beam_zeta_center_star)
+        if isinstance(slices_other_beam_pzeta_center_star, (list, tuple)):
+            slices_other_beam_pzeta_center_star = np.array(slices_other_beam_pzeta_center_star)
+
         if slices_other_beam_zeta_center is not None:
 
-            # Check correct according to z, head at the first position in the arrays
-            assert np.all(slices_other_beam_zeta_center[:-1]
-                            >= slices_other_beam_zeta_center[1:]), (
+            # Check correct according to z, head at the first position in the arrays. Only sort non-empty slices.
+            assert np.all(slices_other_beam_zeta_center[slices_other_beam_num_particles!=0][:-1]
+                            >= slices_other_beam_zeta_center[slices_other_beam_num_particles!=0][1:]), (
                             'slices_other_beam_zeta_center must be sorted from to tail (descending zeta)')
 
             (
@@ -767,6 +895,7 @@ class BeamBeamBiGaussian3D(xt.BeamElement):
                 salpha=self.sin_alpha,
                 calpha=self.cos_alpha,
             )
+
         # User-provided value has priority
         if slices_other_beam_x_center_star is not None:
             self.slices_other_beam_x_center_star = slices_other_beam_x_center_star
@@ -798,24 +927,6 @@ class BeamBeamBiGaussian3D(xt.BeamElement):
         else:
             self.slices_other_beam_pzeta_center_star = self._arr2ctx(pzeta_slices_star)
 
-    # The following properties are generate by this code:
-    ## for nn in 'x px y py zeta pzeta'.split():
-    ##     print(f'''
-    ##     @property
-    ##     def slices_other_beam_{nn}_center(self):
-    ##         (x_slices, px_slices, y_slices, py_slices,
-    ##             zeta_slices, pzeta_slices) = self._inv_boost_slice_centers()
-
-    ##         return self._buffer.context.linked_array_type.from_array(
-    ##             {nn}_slices,
-    ##             mode="readonly")
-
-    ##     @slices_other_beam_{nn}_center.setter
-    ##     def slices_other_beam_{nn}_center(self, value):
-    ##         raise NotImplementedError(
-    ##             "Setting slices_other_beam_{nn}_center is not implemented yet")\n''')
-
-
     @property
     def slices_other_beam_x_center(self):
         (x_slices, px_slices, y_slices, py_slices,
@@ -905,36 +1016,6 @@ class BeamBeamBiGaussian3D(xt.BeamElement):
         raise NotImplementedError(
             "Setting slices_other_beam_pzeta_center is not implemented yet")
 
-    # The following properties are generate by this code:
-    ## for nn, factor in (
-    ##     ('11', '1.'),
-    ##     ('12', 'self.cos_phi'),
-    ##     ('13', '1.'),
-    ##     ('14', 'self.cos_phi'),
-    ##     ('22', '(self.cos_phi * self.cos_phi)'),
-    ##     ('23', 'self.cos_phi'),
-    ##     ('24', '(self.cos_phi * self.cos_phi)'),
-    ##     ('33', '1.'),
-    ##     ('34', 'self.cos_phi'),
-    ##     ('44', '(self.cos_phi * self.cos_phi)')):
-
-    ##     print(f"""
-    ##     @property
-    ##     def slices_other_beam_Sigma_{nn}(self):
-    ##         return self._buffer.context.linked_array_type.from_array(
-    ##               self.slices_other_beam_Sigma_{nn}_star * {factor},
-    ##               mode='setitem_from_container',
-    ##               container=self,
-    ##               container_setitem_name='_Sigma_{nn}_setitem')
-
-    ##     def _Sigma_{nn}_setitem(self, indx, val):
-    ##         self.slices_other_beam_Sigma_{nn}_star[indx] = val / {factor}
-    ##
-    ##     @slices_other_beam_Sigma_{nn}.setter
-    ##     def slices_other_beam_Sigma_{nn}(self, value):
-    ##         self.slices_other_beam_Sigma_{nn}[:] = value\n""")
-
-
     @property
     def slices_other_beam_Sigma_11(self):
         return self._buffer.context.linked_array_type.from_array(
@@ -1207,54 +1288,6 @@ def _python_inv_boost_scalar(x_st, px_st, y_st, py_st, zeta_st, pzeta_st,
 _python_inv_boost = np.vectorize(_python_inv_boost_scalar,
     excluded=("sphi", "cphi", "tphi", "salpha", "calpha"))
 
-class TempSlicer:
-    def __init__(self, bin_edges):
-
-        bin_edges = np.sort(np.array(bin_edges))[::-1]
-        self.bin_edges = bin_edges
-        self.bin_centers = (bin_edges[1:] + bin_edges[:-1]) / 2.0
-        self.num_slices = len(bin_edges) - 1
-
-    def get_slice_indices(self, particles):
-        indices = np.digitize(particles.zeta, self.bin_edges, right=True)
-        indices -= 1 # In digitize, 0 means before the first edge
-        indices[particles.state <=0 ] = -1
-
-        return np.array(indices, dtype=np.int64)
-
-    def assign_slices(self, particles):
-        particles.slice = self.get_slice_indices(particles)
-
-    def compute_moments(self,particles,update_assigned_slices=True):
-        if update_assigned_slices:
-            self.assign_slices(particles)
-
-        slice_moments = np.zeros(self.num_slices*(1+6+10),dtype=float)
-        for i_slice in range(self.num_slices):
-            mask = particles.slice == i_slice
-            slice_moments[i_slice] = len(particles.x[mask])                                                      # nb part
-            slice_moments[self.num_slices+i_slice] = float(particles.x[mask].sum())/slice_moments[i_slice]       # <x>
-            slice_moments[2*self.num_slices+i_slice] = float(particles.px[mask].sum())/slice_moments[i_slice]    # <px>
-            slice_moments[3*self.num_slices+i_slice] = float(particles.y[mask].sum())/slice_moments[i_slice]     # <y>
-            slice_moments[4*self.num_slices+i_slice] = float(particles.py[mask].sum())/slice_moments[i_slice]    # <py>
-            slice_moments[5*self.num_slices+i_slice] = float(particles.zeta[mask].sum())/slice_moments[i_slice]  # <z>
-            slice_moments[6*self.num_slices+i_slice] = float(particles.delta[mask].sum())/slice_moments[i_slice] # <pz> # TODO mhy pzeta doesn't work?
-
-            x_diff = particles.x[mask]-slice_moments[self.num_slices+i_slice]
-            px_diff = particles.px[mask]-slice_moments[2*self.num_slices+i_slice]
-            y_diff = particles.y[mask]-slice_moments[3*self.num_slices+i_slice]
-            py_diff = particles.py[mask]-slice_moments[4*self.num_slices+i_slice]
-            slice_moments[7*self.num_slices+i_slice] = float((x_diff**2).sum())/slice_moments[i_slice]             # Sigma_11
-            slice_moments[8*self.num_slices+i_slice] = float((x_diff*px_diff).sum())/slice_moments[i_slice]      # Sigma_12
-            slice_moments[9*self.num_slices+i_slice] = float((x_diff*y_diff).sum())/slice_moments[i_slice]       # Sigma_13
-            slice_moments[10*self.num_slices+i_slice] = float((x_diff*py_diff).sum())/slice_moments[i_slice]     # Sigma_14
-            slice_moments[11*self.num_slices+i_slice] = float((px_diff**2).sum())/slice_moments[i_slice]           # Sigma_22
-            slice_moments[12*self.num_slices+i_slice] = float((px_diff*y_diff).sum())/slice_moments[i_slice]     # Sigma_23
-            slice_moments[13*self.num_slices+i_slice] = float((px_diff*py_diff).sum())/slice_moments[i_slice]    # Sigma_24
-            slice_moments[14*self.num_slices+i_slice] = float((y_diff**2).sum())/slice_moments[i_slice]            # Sigma_33
-            slice_moments[15*self.num_slices+i_slice] = float((y_diff*py_diff).sum())/slice_moments[i_slice]     # Sigma_34
-            slice_moments[16*self.num_slices+i_slice] = float((py_diff**2).sum())/slice_moments[i_slice]           # Sigma_44
-        return slice_moments
 
 class ConfigForUpdateBeamBeamBiGaussian3D:
 
@@ -1275,5 +1308,3 @@ class ConfigForUpdateBeamBeamBiGaussian3D:
         self._working_on_bunch = None
         self._particles_slice_index = None
 
-
-
